require File.join(File.dirname(__FILE__), 'lib/certapi')
require File.join(File.dirname(__FILE__), 'lib/logger')
require File.join(File.dirname(__FILE__), 'lib/certificate_helpers')

module Certificates
  class Main < Sinatra::Base
    set :root, $path

    helpers CertificateHelpers

    configure do
      mime_type :csr,
        "application/vnd.puppetlabs.certificate-signing-request.metadata.v1+json"
    end

    before do
      cache_control :no_cache
      @logger = CertificateLogger.new
    end

    def batch_operation_delimiter
      ','
    end

    get '/api/v1/csrs' do
      content_type :csr
      CertAPI.csrs.to_json
    end

    get '/api/v1/csrs/:certname' do
      certname = params[:certname]
      if !CertAPI.certnames.include?(certname)
        halt 404, "#{certname} is not a valid certname. The valid certnames are #{CertAPI.certnames.to_json}"
        # It is okay to divulge a list of certnames in the error message, as
        # the user accessing this information must be authenticated to reach
        # this API. If they have permission access to the API, then they have
        # access to this information anyway.
      else
        content_type :csr
        CertAPI.cert_info(certname).to_json
      end
    end

    # sign a CSR
    put '/api/v1/csrs/:certname' do
      certname = params[:certname]
      if !CertAPI.csr_names.include?(certname)
        halt 403, "This request is forbidden because #{certname} does not have a pending certificate request. The pending requests are for #{CertAPI.csr_names.to_json}"
        # This status code could be 403 or 404, but since we are providing the
        # reason for the error it is better left as 403.
      else
        @logger.log_action(user(env), certname, "Add") { CertAPI.add(certname) }
      end
    end

    # sign multiple CSRs
    put '/api/v1/csrs/multi/*' do
      status_code = 200
      params[:splat][0].split(batch_operation_delimiter).each do |certname|
        if !CertAPI.csr_names.include?(certname)
          halt 403, "This request is forbidden because #{certname} does not have a pending certificate request. The pending requests are for #{CertAPI.csr_names.to_json}"
        else
          returned_status_code = @logger.log_action(user(env), certname, "Add") { CertAPI.add(certname) }
          status_code = returned_status_code unless is_successful? returned_status_code
        end
      end
      status_code
    end

    # reject a CSR
    delete '/api/v1/csrs/:certname' do
      certname = params[:certname]
      if !CertAPI.csr_names.include?(certname)
        halt 403, "This request is forbidden because #{certname} does not have a pending certificate request. The pending requests are for #{CertAPI.csr_names.to_json}"
      else
        @logger.log_action(user(env), certname, "Remove") { CertAPI.remove(certname) }
      end
    end

    # reject multiple CSRs
    delete '/api/v1/csrs/multi/*' do
      status_code = 200
      params[:splat][0].split(batch_operation_delimiter).each do |certname|
        if !CertAPI.csr_names.include?(certname)
          halt 403, "This request is forbidden because #{certname} does not have a pending certificate request. The pending requests are for #{CertAPI.csr_names.to_json}"
        else
          returned_status_code = @logger.log_action(user(env), certname, "Remove") { CertAPI.remove(certname) }
          status_code = returned_status_code unless is_successful? returned_status_code
        end
      end
      status_code
    end
  end
end
