define(['underscore', 'jquery', 'sandbox', '../models/request'],
  function(_, $, sandbox, RequestModel) {

  var RequestsCollection = sandbox.mvc.Collection({

    url: 'services/certificates/api/v1/csrs',

    model: RequestModel,

    /**
     * Return a list of requests that have been accepted or rejected.
     * @return {Array}
     */
    completed_requests: function() {
      return this.where({approvable: true, request_open: false});
    },

    /**
     * Return a list of requests that are currently waiting to be accepted or rejected.
     * Note that this also includes requests with DNS alt names, which are not actionable via
     * the certificate manager.
     * @return {Array}
     */
    open_requests: function() {
      return this.where({status: 'pending'})
    },

    /**
     * Return a list of requests that can be accepted or rejected through the GUI.
     * @return {Array}
     */
    actionable_requests: function() {
      return this.where({approvable: true, request_open: true});
    },

    /**
     * Destroy the model instances of all requests that have been actioned.
     */
    clear_requests: function() {
      _.each(this.completed_requests(), function(model) {
        model.destroy();
      });
    },

    /**
     * Accept all open and actionable requests
     * @return {jQuery.promise}
     */
    accept_all: function() {
      return this._do_multi('PUT', function(model) {
                model.set_accepted();
              });

    },

    /**
     * Reject all open and actionable requests
     * @return {jQuery.promise}
     */
    reject_all: function() {
      return this._do_multi('DELETE', function(model) {
                model.set_rejected();
              });

    },

    /**
     * @param request_method string
     * @param model_callback function
     * @return {jQuery.promise}
     * @private
     */
    _do_multi: function(request_method, model_callback) {

      // getting the actionable requests here means we'll always be consistent
      // and call the callback on the same list of requests that we send to the server
      var actionable = this.actionable_requests(),
          deferred = new sandbox.data.deferred();

      if (actionable.length === 0) {
        deferred.resolve();

        return deferred.promise();
      }

      var names = _.map(actionable, function (model) { return model.get('name') }),
          url = 'services/certificates/api/v1/csrs/multi/' + names.join(',');

      deferred.done(function() {
        _.each(actionable, model_callback);
      });

      $.ajax({
        url: url,
        type: request_method,

        success: deferred.resolve,
        error: deferred.reject
      });

      return deferred.promise();
    },

    parse: function(resp) {
      var ret = [];
      var num_nodes = resp.length;
      for (var i = 0; i < num_nodes; i++) {
        var certname = resp[i]['name'];
        var fingerprint = resp[i]['fingerprints']['SHA1'];
        var altnames = resp[i]['dns_alt_names'];
        var has_altname = (altnames.length != 0);

        ret.push({
          name: certname,
          fingerprint: fingerprint,
          dns_alt_name: altnames,
          has_altname: has_altname,
          approvable: !has_altname,
          request_open: !has_altname
        });
      }

      return ret;
    }
  });

  return RequestsCollection;

});
