define(['dom', 'sandbox', './request', '../collections/requests', 'text!../templates/requests.html'],
  function($, sandbox, RequestView, RequestsCollection, template) {

  var EmptyView = sandbox.mvc.ItemView({
    tagName: 'tr',
    template: '<td colspan="99">There are no certificates to accept or reject.</td>'
  });

  /**
   * Selector for the accept/reject buttons in the individual request views.
   * @type {String}
   */
  var ITEMVIEW_BUTTONS_SELECTOR = '.certificates-accept, .certificates-reject';

  var RequestsView = sandbox.mvc.CompositeView({

    itemView: RequestView,
    emptyView: EmptyView,

    itemViewContainer: 'tbody',

    template: template,

    events: {
      'click #certificates-clear-requests': 'clear_requests',
      'click #certificates-accept-all': 'accept_all',
      'click #certificates-reject-all': 'reject_all'
    },

    ui: {
      controls: '#certificates-controls',
      open_count: '.certificates-open-csr-count',
      clear_requests: '#certificates-clear-requests',
      accept_all: '#certificates-accept-all',
      reject_all: '#certificates-reject-all'
    },

    initialize: function(options) {
      this.collection = options.collection || new RequestsCollection;

      // rendering of this view is caused by the collections `reset` event
      this.collection.fetch();

      this.bindTo(this.collection, 'change', this.update_ui_controls, this);
      this.bindTo(this.collection, 'remove', this.update_clear_completed, this);
    },

    onRender: function() {
      this.update_ui_controls();
    },

    /**
     * Keep the total request count and accept/reject all button count in sync.
     */
    update_ui_controls: function() {
      this.update_open_count();
      this.update_clear_completed();
      this.update_action_buttons();
    },

    /**
     * Remove all completed requests from the UI
     */
    clear_requests: function() {
      this.collection.clear_requests();
    },

    reject_all: function() {
      this.disable_controls();

      var reenable = $.proxy(this.enable_controls, this),
          update = $.proxy(this.update_ui_controls, this);

      this.collection.reject_all().done([reenable, update])
                                  .fail(reenable);
    },

    accept_all: function() {
      this.disable_controls();

      var reenable = $.proxy(this.enable_controls, this),
          update = $.proxy(this.update_ui_controls, this);

      this.collection.accept_all().done([reenable, update])
                                  .fail(reenable);
    },

    disable_controls: function() {
      this.ui.controls.find('button').attr('disabled', 'disabled');

      this.disable_itemview_controls();
    },

    /**
     * Disable the accept/reject buttons for the individual request views
     */
    disable_itemview_controls: function() {
      this.$(ITEMVIEW_BUTTONS_SELECTOR).attr('disabled', 'disabled');
    },

    enable_controls: function() {
      this.ui.controls.find('button').removeAttr('disabled');

      this.enable_itemview_controls();
    },

    /**
     * Enable the accept/reject buttons for the individual request views
     */
    enable_itemview_controls: function() {
      this.$(ITEMVIEW_BUTTONS_SELECTOR).removeAttr('disabled');
    },

    update_open_count: function() {
      var pending = this.collection.open_requests().length,
          text;

      if (pending === 1) {
        text = 'There is 1 node requesting to join your site.';
      } else if (pending > 1) {
        text = 'There are ' + pending + ' nodes requesting to join your site.';
      } else {
        text = 'There are no nodes requesting to join your site.';
      }

      this.ui.open_count.text(text);
    },

    update_clear_completed: function() {
      var complete = this.collection.completed_requests().length,
          text;

      if (complete === 1) {
        text = 'Clear 1 accepted/rejected request';
        this.ui.clear_requests.removeAttr('disabled');
      } else if (complete > 1) {
        text = 'Clear ' + complete + ' accepted/rejected requests';
        this.ui.clear_requests.removeAttr('disabled');
      } else {
        text = 'There are no accepted/rejected requests to clear';
        this.ui.clear_requests.attr('disabled', 'disabled');
      }

      this.ui.clear_requests.text(text);
    },

    update_action_buttons: function() {
      var actionable = this.collection.actionable_requests().length;

      if (actionable === 0) {
        this.ui.accept_all.attr('disabled', 'disabled');
        this.ui.reject_all.attr('disabled', 'disabled');
      } else {
        this.ui.accept_all.removeAttr('disabled');
        this.ui.reject_all.removeAttr('disabled');
      }
    }
  });

  return RequestsView;
});